# logger_config.py
from psrx_class import devices
from psrx_class import logging
from datetime import datetime

def setup_logging():
    # Get the current date and time in the desired format
    current_datetime = datetime.now()
    formatted_datetime = current_datetime.strftime('%d-%m-%yT%H-%M')

    # Set the log file path using the formatted datetime
    log_file_path = rf'C:\{devices.package_name}\logs\{formatted_datetime}.log'

    # Configure logging
    logging.basicConfig(
        filename=log_file_path,
        level=logging.DEBUG,
        format='%(asctime)s - %(levelname)s - %(message)s'
    )



from psrx_class import save_excel_def

def get_configured_policy(file_content=None):
    data = save_excel_def.save_excel(type_='policy',file_content=file_content, file_name='no name',onlyData='data')
    return data


#['zone-based', 'Management_Platform_FW', 'DC_FW_to_ACI_North-South',
#  'AV_UTL_DNS_Access1',
#  'Avamar_Vault\nAvamar_Vault_ADM\n',
#  'DNS\n', 'TCP_53\nUDP_53\njunos-icmp-all\n',
#  'any\n', 'none\n', 'permit\ncount\n', 'idp-policy Recommended', 'session-close']

def get_all_policies(dev=None,file_content=None):
    all_policies = get_configured_policy(file_content=file_content)
    add = {
        'Avamar_Vault':'192'
    }

    source = input('Source : ')
    destination = input('Destination : ')
    app = input('Application : ')

    


from prompt_toolkit import prompt
from prompt_toolkit.completion import NestedCompleter
import re
from psrx_class import devices
from psrx_class import _configurations_
from psrx_class import save_excel_def
from psrx_class import _main_functions_
from psrx_class import match_policy
from jnpr.junos.utils.config import Config
from psrx_class.logger_config import setup_logging
import logging

# Call the setup_logging function from logger_config
setup_logging()

module_name = __name__.split('.')[-1]


class hierarchy: # A hierarchy for commands


    def Crossroad(self):
        # Define a hierarchy of commands using a nested completer.
        completer = NestedCompleter.from_nested_dict({
            'local': None,
            'netconf': None,
            'exit': None,
        })
        
        return completer

    def local(self):

        _Backups_files = devices.loacls_Backup()

        completer_local = NestedCompleter.from_nested_dict({})

        completer_local_dict = {}
        
        for item in _Backups_files:
            completer_local_dict[item] = None
        
        completer_local_dict['return'] = None
        
        completer_local = NestedCompleter.from_nested_dict(completer_local_dict)
        
        return completer_local


    def netconf(self):
        
        FW_name = []
        FW_info = []
        
        for key, value in devices.FWs.items():
            FW_name.append(key)
            FW_info.append(value)

        completer_netconf = NestedCompleter.from_nested_dict({})

        completer_netconf_dict = {}

        for item in FW_name:
            completer_netconf_dict[item] = None

        
        completer_netconf_dict['return'] = None
        
        completer_netconf = NestedCompleter.from_nested_dict(completer_netconf_dict)
        
        return completer_netconf
            
    def commands(self):   
        completer_netconf_fw = NestedCompleter.from_nested_dict({
            'push': {
            'policy': None,
            'full-set-commands': None,
            'address': None,
            'application': None,
            },
            'get': {
             'match-policies':None,
            },
            'show': {
            'address': {
            'ip': None,
            'zone': None,
            'interface': None,
            },
            'application': None,
            },
            'save-as-excel': {
            'all-configuration': {
            'select-node':None
            },
            'policies': None,
            'logical-policy': None,
            'address': None,
            'address-set': None,
            'application': None,
            'application-set': None,
            },
            'close-session': None,
        })
        
        return completer_netconf_fw
            
            
# Define a function to handle the user's input.

def select_multi_FW(ff):
    coms_obj = hierarchy()
    files_contents = []
    files_name = []
    try:
        files_content = devices.open_local_backup_file(ff)
    except Exception as e:
        print(f"Invalid command: {ff}")
        
    files_contents.append(files_content)
    files_name.append(ff)
    #choosing file name     
    while True:
        uf = prompt(' > ',completer=coms_obj.local()).strip()
        if re.match(uf,r'\s+$'):
                break
        elif re.match(uf,r'return$'):
                return 'return' 
                
        files_content = devices.open_local_backup_file(uf)
        
        if not(files_content in files_contents):
            files_contents.append(files_content)
            files_name.append(uf)
        else:
            print(f' !> {uf} Already added.')
            continue
        
    return [files_contents,files_name]
def netconf(mode):
    coms_obj = hierarchy()
    logging.debug(f' {module_name} This is a debug message in some_module')
    logging.info(f' {module_name} This is an info message in some_module')
    while True:
        
        if mode == 'netconf' :
            # Cridential as list for the selected device [option,'192.168.1.201','root','Password']
            cri = devices.netconf() 
            
            # open a connection with the device and start a NETCONF session
            if len(cri) == 4:
                cridential_as_list = [cri[1],cri[2],cri[3]]
                netconf_name = f'{cri[0]}@{cri[2]}'
            elif len(cri) == 3:
                cridential_as_list = cri
                netconf_name = cri[1]
            dev = _main_functions_.open_session(cridential_as_list)
            
            if dev == 'retry':continue
                        
            # make an object from configuration utility
            cu = Config(dev, mode='private')
            
            uf = prompt(f'CLI > {mode} > {netconf_name} >').strip()
                
        elif mode == 'local':
                
            #choosing file name
            uf = prompt(f'CLI > {mode} >', completer=coms_obj.local()).strip()

        if uf == 'return': return 'return' 
        
        #intialize session
        if uf != 'return':
        
 
            if mode == 'local':
                
                try:
                
                    file_content = devices.open_local_backup_file(uf)
                
                except Exception as e:
                    print(f"Invalid command: {uf}")
                    continue
            
        while True:

            #choosing commands
            u_c_f_n3 = prompt(f'CLI > {mode} > {uf} >', completer=coms_obj.commands()).strip()
            
            #push policy
            if re.match(r'push\s+policy$',u_c_f_n3):
                if mode == 'netconf' :
                    if _configurations_.load_policy(cu,dev): continue
                elif mode == 'local':
                    if _configurations_.load_policy(file_content=file_content): continue
                    continue
                    
            #push full-set-commands
            if re.match(r'push\s+full-set-commands$',u_c_f_n3):
                if mode == 'netconf':
                    if _configurations_.load_full_set_commands(cu): continue
                elif mode == 'local':
                    print('THIS COMMAND NOT CONFIGURED YET.')
                    continue
                
            #push address
            if re.match(r'push\s+address$',u_c_f_n3):
                if mode == 'netconf' :
                    if _configurations_.load_adresses(cu,dev): continue
                elif mode == 'local':
                    if _configurations_.load_adresses(file_content=file_content): continue
                    continue
                
            #push application
            if re.match(r'push\s+application$',u_c_f_n3):
                if mode == 'netconf' :
                    if _configurations_.load_application(cu,dev): continue
                elif mode == 'local':
                    if _configurations_.load_application(file_content=file_content): continue
                    continue

             #get match-policies
            if re.match(r'get\s+match-policies$',u_c_f_n3):
                if mode == 'netconf' :
                    if match_policy.get_all_policies(dev=dev): continue
                elif mode == 'local':
                    if match_policy.get_all_policies(file_content=file_content): continue
                    continue
                    
            #show address ip
            if re.match(r'show\s+address\s+ip$',u_c_f_n3):
                if mode == 'netconf' :
                    print('THIS COMMAND NOT CONFIGURED YET.')
                elif mode == 'local':
                    print('THIS COMMAND NOT CONFIGURED YET.')
                    continue
                    
            #show address zone
            if re.match(r'show\s+address\s+zone$',u_c_f_n3):
                if mode == 'netconf' :
                    print('THIS COMMAND NOT CONFIGURED YET.')
                elif mode == 'local':
                    print('THIS COMMAND NOT CONFIGURED YET.')
                    continue
                    
            #show address interface
            if re.match(r'show\s+address\s+interface$',u_c_f_n3):
                if mode == 'netconf' :
                    print('THIS COMMAND NOT CONFIGURED YET.')
                elif mode == 'local':
                    print('THIS COMMAND NOT CONFIGURED YET.')
                    continue
                    
            #show application
            if re.match(r'show\s+application$',u_c_f_n3):
                if mode == 'netconf' :
                    print('THIS COMMAND NOT CONFIGURED YET.')
                elif mode == 'local':
                    print('THIS COMMAND NOT CONFIGURED YET.')
                    continue
                    
            #save-as-excel all-configuration select-node
            if re.match(r'save-as-excel\s+all-configuration\s+select-node$',u_c_f_n3):
                if mode == 'netconf' :
                    print('THIS COMMAND NOT AVILABLE FOR NETCONF TRY WITHOUT select-node ')
                elif mode == 'local':
                    print('THIS COMMAND NOT CONFIGURED YET.')
                    files_contents = select_multi_FW(uf)
                    # if files_contents != 'return':
                        # if save_excel_def.save_excel(type_='select-node',file_content=files_contents[0],file_name=files_contents[1]): continue
                        # continue
                    # else:
                        # continue
                    
            #save-as-excel all-configuration
            if re.match(r'save-as-excel\s+all-configuration$',u_c_f_n3):
                if mode == 'netconf' :
                    print('THIS COMMAND NOT CONFIGURED YET.')
                elif mode == 'local':
                    if save_excel_def.save_excel(type_='all-configuration',file_content=file_content,file_name=uf): continue
                    continue
            
            #save-as-excel policies
            if re.match(r'save-as-excel\s+policies$',u_c_f_n3):
                if mode == 'netconf' :
                    print('THIS COMMAND NOT CONFIGURED YET.')
                elif mode == 'local':
                    if save_excel_def.save_excel(type_='policy',file_content=file_content,file_name=uf): continue
                    continue
                    
            #save-as-excel policies
            if re.match(r'save-as-excel\s+logical-policy$',u_c_f_n3):
                if mode == 'netconf' :
                    print('THIS COMMAND NOT CONFIGURED YET.')
                elif mode == 'local':
                    if save_excel_def.save_excel(type_='logical_policy',file_content=file_content,file_name=uf): continue
                    continue
                    
            #save-as-excel address
            if re.match(r'save-as-excel\s+address$',u_c_f_n3):
                if mode == 'netconf' :
                    print('THIS COMMAND NOT CONFIGURED YET.')
                elif mode == 'local':
                    if save_excel_def.save_excel(type_='address',file_content=file_content,file_name=uf): continue
                    continue
                    
            #save-as-excel address-set
            if re.match(r'save-as-excel\s+address-set$',u_c_f_n3):
                if mode == 'netconf' :
                    print('THIS COMMAND NOT CONFIGURED YET.')
                elif mode == 'local':
                    if save_excel_def.save_excel(type_='address-set',file_content=file_content,file_name=uf): continue
                    continue
            
            #save-as-excel application
            if re.match(r'save-as-excel\s+application$',u_c_f_n3):
                if mode == 'netconf' :
                    print('THIS COMMAND NOT CONFIGURED YET.')
                elif mode == 'local':
                    if save_excel_def.save_excel(type_='application',file_content=file_content,file_name=uf): continue
                    continue
                    
            #save-as-excel application-set
            if re.match(r'save-as-excel\s+application-set$',u_c_f_n3):
                if mode == 'netconf' :
                    print('THIS COMMAND NOT CONFIGURED YET.')
                elif mode == 'local':
                    if save_excel_def.save_excel(type_='application-set',file_content=file_content,file_name=uf): continue
                    continue
                    
            #close-session
            if re.match(r'close-session$',u_c_f_n3):
                if mode == 'netconf' :
                    _main_functions_.session_close(dev)
                break

            else:
                print(f"Invalid command: {u_c_f_n3}")
    

def exiting():
    print("Exiting the CLI.")
    return 'exit'  # Exit the CLI loop    
    
def handle_input(text):

        if text.strip() == 'exit':
            return exiting()
        
        
        elif text.strip() == 'local':
            return netconf('local')
        
        
        elif text.strip() == 'netconf':
            return netconf('netconf')
            
        else:
           print(f"Invalid command: {text}")


    
def main_cli_loop():
    while True:
        main_com = hierarchy()
        
        user_input = prompt('CLI> ', completer=main_com.Crossroad())
        
        main_input = handle_input(user_input)
        if main_input == 'exit':
            break
        

    


import re
import ipaddress
from termcolor import colored
from psrx_class import _configurations_

address_description = 'Address Pushed through Netconf'




excelude_address = [
    'address',
    'address-set',
    'address range',
    'address-range',
    'address group',
    'address-group',
    'address_group',
    ''
    ]



  
def address_address_set(dev, input_any_address_to_configure , file_content = None):
    invalid_range = []
    already_existed=[]
    invalid_address_set = []
    commands=[]
    already_existed=[]
    duplicated_in_commands = []
    duplicated_ip=[]
    duplicated_ip_in_commands = []
    wrong_inputs=[]
    
    lines = input_any_address_to_configure.strip().split('\n')
     # [[each line.split], [each line.split], [each line.split]]
    result = [line.split('\t') for line in lines]

    for each_line1 in result:
    
# if >>  For address and address-range >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>.
        if len(each_line1) >= 4 and each_line1[3] != '':
            each_line = [element.replace(' ', '_').replace('&', 'And').replace('(', '').replace(')', '').replace('\r','').strip() for element in each_line1]

            # if no cloumn 5 add column 5 = ''
            if len(each_line) == 4:
                each_line.append('') 
            
            
            is_ip3 = network_ip(each_line[3], each_line)
            if isinstance(is_ip3, list):    
                if is_ip3[0] == 'error':
                    wrong_inputs.append(is_ip3[1])
                    continue
            if each_line[4] != '' :
                is_ip4 = network_ip(each_line[4], each_line)
                if isinstance(is_ip4, list):
                    if is_ip4[0] == 'error':
                        wrong_inputs.append(is_ip4[1])
                        continue
                
            
            
            # match4 is a prefix in column 5 in excel sheet and must to be 32
            match3 = ip_prefix(each_line[3])
            match4 = ip_prefix(each_line[4])
            
            # no_slash3 is ip with no prefix in column 4 in excel sheet 
            # no_slash4 is ip with no prefix in column 5 in excel sheet 
            no_slash3 = each_line[3].split('/')[0] if '/' in each_line[3] else each_line[3] 
            no_slash4 = each_line[4].split('/')[0] if '/' in each_line[4] else each_line[4]
            
            
            if not(each_line[3].find('/') != -1) and is_valid_ip(each_line[3]):
                each_line[3] = each_line[3] + '/32'

            if not(each_line[4].find('/') != -1) and is_valid_ip(each_line[4]):
                each_line[4] = each_line[4] + '/32'
 
            


            if dev != None:
                retuen_match_in_list = _configurations_.retrive_configurations('address' , each_line[1], dev=dev)
            if file_content != None:
                retuen_match_in_list = _configurations_.retrive_configurations('address' , each_line[1], file_content=file_content)
            #==================== For Address Only ==================================================================================================
            # each_line[3] is the value in column 4 in excel sheet 
            # each_line[4] is the value in column 5 in excel sheet
            if each_line[4] == '' or match4 == 32 or no_slash3 == no_slash4 :
                if dev != None:
                    matched_ip_list = _configurations_.retrive_configurations('address-ip' , each_line[3], dev=dev)
                if file_content != None:
                    matched_ip_list = _configurations_.retrive_configurations('address-ip' , each_line[3], file_content=file_content) 
                if each_line[2] != '' and  each_line[4] == '': # ['address', word, description, ip , '']
                    if not( f'set security address-book global description {each_line[2]} address {each_line[1]} {each_line[3]}' in commands):
                        if len(retuen_match_in_list) == 0:
                                if len(matched_ip_list) == 0 and not(f'set security address-book global description {each_line[2]} address {each_line[1]} {each_line[3]}' in commands):
                                    for comm in commands:
                                        if  re.search(r'security address-book global address .* {}'.format(each_line[3]),comm):
                                            words = comm.split()
                                            duplicated_ip_in_commands.append(yellow('► WARNING ► Duplicated IP [') + yellow(each_line[3])+yellow('] Found Under ▼▼ Paste your set commands ▼▼.With Name [') +m(words[5])+ yellow('].Still configured as valid and associated with name [') + m(each_line[1]) + yellow('].'))
                                    commands.append(f'set security address-book global description {b(each_line[2])} address {m(each_line[1])} {yellow(each_line[3])}')
                                    continue
                                else:
                                    commands.append(f'set security address-book global description {b(each_line[2])} address {m(each_line[1])} {yellow(each_line[3])}')
                                    duplicated_ip.append(yellow('► WARNING ► ') + 'Duplicated IP Found [' + yellow(each_line[3]) + '] This IP already existed and associated with different name.Still configured as valid and associated with name [' + m(each_line[1]) + '].')
                                    duplicated_with = yellow('► Duplicated With ► \n') + '\n'.join(matched_ip_list)
                                    duplicated_ip.append(duplicated_with)
                                    continue
                        else:
                            already_existed.append(f'The Address [ {m(each_line[1])} ] Already existed. No action had taken.')
                            continue
                    else:
                        duplicated_in_commands.append(f'The Address [ {m(each_line[1])} ] Is duplicated and done once. No action had taken.')
                        continue
                    
                elif each_line[2] != '' and  each_line[4] != '': # ['address', word, description, ip , ip/prefix]
                    if not(f'set security address-book global description {each_line[2]} address {each_line[1]} {each_line[4]}' in commands):
                        if len(retuen_match_in_list) == 0:    
                            if len(matched_ip_list) == 0 and not(f'set security address-book global description {each_line[2]} address {each_line[1]} {each_line[4]}' in commands):
                                for comm in commands:
                                        if  re.search(r'security address-book global address .* {}'.format(each_line[4]),comm):
                                            words = comm.split()
                                            duplicated_ip_in_commands.append(yellow('► WARNING ► ') + 'Duplicated IP [' + yellow(each_line[4])+ '] Found Under ▼▼ Paste your set commands ▼▼.With Name [' +m(words[5])+ '].Still configured as valid and associated with name [' + m(each_line[1]) + '].')
                                commands.append(f'set security address-book global description {b(each_line[2])} address {m(each_line[1])} {yellow(each_line[4])}')
                                continue
                            else:
                                commands.append(f'set security address-book global description {b(each_line[2])} address {m(each_line[1])} {yellow(each_line[4])}')
                                duplicated_ip.append(yellow('► WARNING ► ') + 'Duplicated IP Found [' + m(each_line[4]) + '] This IP already existed and associated with different name.Still configured as valid and associated with name [' + m(each_line[1]) + '].')
                                duplicated_with = yellow('► Duplicated With ► \n') + '\n'.join(matched_ip_list)
                                duplicated_ip.append(duplicated_with)
                                continue    
                        else:
                            already_existed.append(f'The Address [ {m(each_line[1])} ] Already existed. No action had taken.')
                            continue
                    else:
                        duplicated_in_commands.append(f'The Address [ {m(each_line[1])} ] Is duplicated and done once. No action had taken.')
                        continue
                        
                elif each_line[2] == '' and  each_line[4] == '': # ['address', word, '', ip , ''] 
                    if not(f'set security address-book global address {each_line[1]} {each_line[3]}' in commands):
                        if len(retuen_match_in_list) == 0:
                            if len(matched_ip_list) == 0 and not(f'set security address-book global address {each_line[1]} {each_line[3]}' in commands):
                                for comm in commands:
                                        if  re.search(r'security address-book global address .* {}'.format(each_line[3]),comm):
                                            words = comm.split()
                                            duplicated_ip_in_commands.append(yellow('► WARNING ► ') + 'Duplicated IP [' + yellow(each_line[3]) +  '] Found Under ▼▼ Paste your set commands ▼▼.With Name [' +m(words[5])+ '].Still configured as valid and associated with name [' + m(each_line[1]) + '].')
                                commands.append(f'set security address-book global address {m(each_line[1])} {yellow(each_line[3])}') 
                                continue
                            else:
                                commands.append(f'set security address-book global address {m(each_line[1])} {yellow(each_line[3])}') 
                                duplicated_ip.append(yellow('► WARNING ► ') + 'Duplicated IP Found [' + m(each_line[3]) + '] This IP already existed and associated with different name.Still configured as valid and associated with name [' + m(each_line[1]) + '].')
                                duplicated_with = yellow('► Duplicated With ► \n') + '\n'.join(matched_ip_list)
                                duplicated_ip.append(duplicated_with)
                                continue  
                        else:
                            already_existed.append(f'The Address [ {m(each_line[1])} ] Already existed. No action had taken.')
                            continue
                    else:
                        duplicated_in_commands.append(f'The Address [ {m(each_line[1])} ] Is duplicated and done once. No action had taken.')
                        continue
                    
                elif each_line[2] == '' and  each_line[4] != '': # ['address', word, '', ip , ip/prefix]
                    if not(f'set security address-book global address {each_line[1]} {each_line[4]}' in commands):
                        if len(retuen_match_in_list) == 0:  
                            if len(matched_ip_list) == 0 and not(f'set security address-book global address {each_line[1]} {each_line[4]}' in commands):
                                for comm in commands:
                                        if  re.search(r'security address-book global address .* {}'.format(each_line[4]),comm):
                                            words = comm.split()
                                            duplicated_ip_in_commands.append(yellow('► WARNING ► ') + 'Duplicated IP [' + yellow(each_line[4]) + '] Found Under ▼▼ Paste your set commands ▼▼.With Name [' + m(words[5]) + '].Still configured as valid and associated with name [' + m(each_line[1]) + '].')
                                commands.append(f'set security address-book global address {m(each_line[1])} {yellow(each_line[4])}')
                                continue
                            else:
                                commands.append(f'set security address-book global address {m(each_line[1])} {yellow(each_line[4])}')
                                duplicated_ip.append(yellow('► WARNING ► ') + 'Duplicated IP Found [' + m(each_line[4]) + '] This IP already existed and associated with different name.Still configured as valid and associated with name [' + m(each_line[1]) + '].')
                                duplicated_with = yellow('► Duplicated With ► \n') + '\n'.join(matched_ip_list)
                                duplicated_ip.append(duplicated_with)
                                continue  
                        else:
                            already_existed.append(f'The Address [ {m(each_line[1])} ] Already existed. No action had taken.')
                            continue
                    else:
                        duplicated_in_commands.append(f'The Address [ {m(each_line[1])} ] Is duplicated and done once. No action had taken.')
                        continue
                    
                        
                    
                 
                    
                        
                    
            #==================== For Address-range Only ==================================================================================================
            if len(each_line) >= 5 and each_line[4] != '':
                end_ip =network_ip(each_line[4], each_line)
                start_ip =network_ip(each_line[3], each_line)
                
                if end_ip == 'error': continue
                if start_ip == 'error': continue
                
                ip_range = int(end_ip) - int(start_ip)
                if dev != None:
                    matched_ip_list = _configurations_.retrive_configurations('address-range-ip' , [start_ip,end_ip], dev=dev)
                if file_content != None:
                    matched_ip_list = _configurations_.retrive_configurations('address-range-ip' , [start_ip,end_ip], file_content=file_content) 
                if ip_range > 0:
                    if each_line[2] != '': # ['address-set', word, description, ip, ip]
                        if not( f'set security address-book global address {each_line[1]} description {each_line[2]} range-address {start_ip} to {end_ip}' in commands):
                            if len(retuen_match_in_list) == 0:  
                                if len(matched_ip_list) == 0:
                                    for comm in commands:
                                        if  re.search(r'set security address-book global address .* range-address {} to {}'.format(start_ip,end_ip),comm):
                                            words = comm.split()
                                            duplicated_ip_in_commands.append(yellow('► WARNING ► ') + 'Duplicated Range-IP [' + yellow(start_ip) + g(' to ') + yellow(end_ip) + '] Found Under ▼▼ Paste your set commands ▼▼.With Name [' + m(words[5]) + '].Still configured as valid and associated with name [' + m(each_line[1]) + '].')
                                    commands.append(f'set security address-book global address {m(each_line[1])} description {b(each_line[2])} range-address {yellow(start_ip)} to {yellow(end_ip)}')
                                    continue
                                else:
                                    commands.append(f'set security address-book global address {m(each_line[1])} description {b(each_line[2])} range-address {yellow(start_ip)} to {yellow(end_ip)}')
                                    duplicated_ip.append(yellow('► WARNING ► ') + 'Duplicated Range-IP Found [' + yellow(start_ip) + g(' to ') + yellow(end_ip) + '] This IP already existed and associated with different name.Still configured as valid and associated with name [' +  m(each_line[1]) + '].')
                                    duplicated_with = yellow('► Duplicated With ► \n') + '\n'.join(matched_ip_list)
                                    duplicated_ip.append(duplicated_with)
                                    continue
                            else:
                                already_existed.append(f'The Address [{m(each_line[1])}] Already existed. No action had taken.')
                                continue
                        else:
                            duplicated_in_commands.append(f'The Address [{m(each_line[1])}] Is duplicated and done once. No action had taken.')
                            continue
                        
                    
                    elif each_line[2] == '': # ['address', word, '', ip , ip]
                        if not(f'set security address-book global address {each_line[1]} range-address {start_ip} to {end_ip}' in commands):
                            if len(retuen_match_in_list) == 0:
                                if len(matched_ip_list) == 0:
                                    for comm in commands:
                                        if  re.search(r'set security address-book global address .* range-address {} to {}'.format(start_ip,end_ip),comm):
                                            words = comm.split()
                                            duplicated_ip_in_commands.append(yellow('► WARNING ► ') + 'Duplicated Range-IP [' + yellow(start_ip) + g(' to ') + yellow(end_ip) + '] Found Under ▼▼ Paste your set commands ▼▼.With Name [' + m(words[5]) + '].Still configured as valid and associated with name [' + m(each_line[1]) + '].')
                                    commands.append(f'set security address-book global address {m(each_line[1])} range-address {yellow(start_ip)} to {yellow(end_ip)}')
                                    continue
                                else:
                                    commands.append(f'set security address-book global address {m(each_line[1])} description {b(each_line[2])} range-address {yellow(start_ip)} to {yellow(end_ip)}')
                                    duplicated_ip.append(yellow('► WARNING ► ') + 'Duplicated Range-IP Found [' + yellow(start_ip) + g(' to ') + yellow(end_ip) + '] This IP already existed and associated with different name.Still configured as valid and associated with name [' +  m(each_line[1]) + '].')
                                    duplicated_with = yellow('► Duplicated With ► \n') + '\n'.join(matched_ip_list)
                                    duplicated_ip.append(duplicated_with)
                                    continue
                            else:
                                already_existed.append(f'The Address [ {m(each_line[1])} ] Already existed. No action had taken.')
                                continue
                        else:
                            duplicated_in_commands.append(f'The Address [ {m(each_line[1])} ] Is duplicated and done once. No action had taken.')
                            continue    

                elif ip_range < 0:
                    invalid_range.append(yellow("Error: ") +  'Invalid IPv4 address Range: \b' + yellow(each_line[3]) + ' to ' + yellow(each_line[4]) + "no action had taken.")
                    continue
                    
  # if >>  For address and address-set >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>.  
  
        #inilaize each_line1 add more elements '' if have less than 4
        if len(each_line1) < 4:
            for more_element in range(4 - len(each_line1)):
                each_line1.append('')
        
        each_line = [element.replace(' ', '_').replace('&', 'And').replace('(', '').replace(')', '').replace('\r','').strip() for element in each_line1]
        if each_line[3] == '':
            
            

            # send column 2 value to retrived data > return true or false
            
            if dev != None:
            
                is_excist_address = _configurations_.retrive_configurations( 'address', each_line[1], is_exist= 'yes', dev = dev)
                retuen_match_in_list = _configurations_.retrive_configurations('address-set' , [each_line[0],each_line[1]], is_exist= 'yes', dev=dev)
                
            if file_content != None:
                
                is_excist_address = _configurations_.retrive_configurations( 'address', each_line[1], is_exist= 'yes',file_content=file_content)
                retuen_match_in_list = _configurations_.retrive_configurations('address-set' , [each_line[0],each_line[1]], file_content=file_content)
 
            if is_excist_address and not(each_line[0].lower() in [s.lower() for s in excelude_address]):   # ['address-set', address, '', ''] 
                    if len(retuen_match_in_list) == 0:
                        if not(f'set security address-book global address-set {each_line[0]} address {each_line[1]}' in commands):
                            commands.append(f'set security address-book global address-set {m(each_line[0])} address {m(each_line[1])}')
                            continue
                        else:
                            duplicated_in_commands.append(f'The address [{m(each_line[1])}] Is duplicated and done once. No action had taken.')
                            continue
                    else:
                        already_existed.append(f'The address [{m(each_line[1])}] Already existed in address-set called [ {m(each_line[0])}]. No action had taken.')
                        continue    
            else:

                invalid_address_set.append(f'The address [{m(each_line[1])}] not exist. So you can not add it to address-set [{m(each_line[0])}]. No action had taken.')
                continue
                
    return [commands, wrong_inputs, invalid_range, invalid_address_set, address_description,duplicated_in_commands ,already_existed,duplicated_ip,duplicated_ip_in_commands]

def network_ip(ip_str,invalid_line):
    
    try:
    
        ip_network = ipaddress.IPv4Network(ip_str, strict=False)
        
        # Extract the IPv4 address without the subnet
        ipv4_address = ip_network.network_address
        return ipv4_address
        
    except Exception as e:
        wrong_inputs = yellow("Error: ") + 'Invalid characters in the IPv4 address: \b ' + ' | '.join([x for x in invalid_line if x !='']) + " no action had taken."
        return ['error',wrong_inputs]
        
        
def ip_prefix(ip_str):
        if ip_str.find('/') != -1 :
            ipv4_prefix = ip_str.split('/')[1]
            return ipv4_prefix
            
      

    
def is_valid_ip(ip_str):
    try:
        ip = ipaddress.ip_address(ip_str)
        return True
    except ValueError:
        return False
    
def m(word):
    styled_text = colored(word, "magenta", attrs=["bold"])
    return styled_text
def c(word):
    styled_text = colored(word, "cyan", attrs=["bold"])
    return styled_text
def b(word):
    styled_text = colored(word, "blue", attrs=["bold"])
    return styled_text
def yellow(word):
    styled_text = colored(word, "yellow", attrs=["bold"])
    return styled_text
def g(word):
    styled_text = colored(word, "green", attrs=["bold"])
    return styled_text


import os
from pathlib import Path
import getpass
import ipaddress

# Use setup.name or any other attribute you need
package_name = 'psrx'
files = ['Backup','Excel Outputs','logs']

# Specify the path of the new folder


for x in files:
    # Specify the path of the new folder
    new_folder_path = Path(f"C:\{package_name}\{x}")  # Update the path as needed

    # Create the new folder
    new_folder_path.mkdir(parents=True, exist_ok=True)

# Define the path to the Backup folder
backup_folder = f"C:\{package_name}\Backup"  # Adjust the path here



# List files in the Backup folder

backup_files = os.listdir(backup_folder)


    
def netconf():
    option = input('Enter box name (Optional): ')
    
    while True:
        ssh = input('Enter IP to connect: ')
        if is_valid_ipv4(ssh):
            break
        else:
            print('Invalid IP, Please try again.')
    user = input('Enter your username: ')
    password = getpass.getpass("Enter your password: ")
    if option == '' or option == None:
        return [ssh,user,password]
    else:
        return [option,ssh,user,password]

def is_valid_ipv4(ip):
    try:
        ipaddress.IPv4Address(ip)
        return True
    except ipaddress.AddressValueError:
        return False
    

def loacls_Backup():

    chosen_files = []
    for i, file in enumerate(backup_files):
        chosen_files.append(file[:-4])
    return chosen_files

        
 
def open_local_backup_file(file_name):
    chosen_file = os.path.join(backup_folder, f'{file_name}.txt')

    # Search for the word 'World!' in the chosen file
    with open(chosen_file, "r") as file:
    
        lines = file.read()
        
        file_contents = lines.strip().split('\n')
        
        return file_contents
            